# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: graphsenselib/ingest/tron/grpc/core/contract.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from graphsenselib.ingest.tron.grpc.core import common_pb2 as graphsenselib_dot_ingest_dot_tron_dot_grpc_dot_core_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2graphsenselib/ingest/tron/grpc/core/contract.proto\x12\x08protocol\x1a\x30graphsenselib/ingest/tron/grpc/core/common.proto\"l\n\x15\x41\x63\x63ountCreateContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x17\n\x0f\x61\x63\x63ount_address\x18\x02 \x01(\x0c\x12#\n\x04type\x18\x03 \x01(\x0e\x32\x15.protocol.AccountType\"D\n\x15\x41\x63\x63ountUpdateContract\x12\x14\n\x0c\x61\x63\x63ount_name\x18\x01 \x01(\x0c\x12\x15\n\rowner_address\x18\x02 \x01(\x0c\"A\n\x14SetAccountIdContract\x12\x12\n\naccount_id\x18\x01 \x01(\x0c\x12\x15\n\rowner_address\x18\x02 \x01(\x0c\"\xab\x01\n\x1f\x41\x63\x63ountPermissionUpdateContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12#\n\x05owner\x18\x02 \x01(\x0b\x32\x14.protocol.Permission\x12%\n\x07witness\x18\x03 \x01(\x0b\x32\x14.protocol.Permission\x12%\n\x07\x61\x63tives\x18\x04 \x03(\x0b\x32\x14.protocol.Permission\"\x90\x04\n\x12\x41ssetIssueContract\x12\n\n\x02id\x18) \x01(\t\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x0c\n\x04name\x18\x02 \x01(\x0c\x12\x0c\n\x04\x61\x62\x62r\x18\x03 \x01(\x0c\x12\x14\n\x0ctotal_supply\x18\x04 \x01(\x03\x12@\n\rfrozen_supply\x18\x05 \x03(\x0b\x32).protocol.AssetIssueContract.FrozenSupply\x12\x0f\n\x07trx_num\x18\x06 \x01(\x05\x12\x11\n\tprecision\x18\x07 \x01(\x05\x12\x0b\n\x03num\x18\x08 \x01(\x05\x12\x12\n\nstart_time\x18\t \x01(\x03\x12\x10\n\x08\x65nd_time\x18\n \x01(\x03\x12\r\n\x05order\x18\x0b \x01(\x03\x12\x12\n\nvote_score\x18\x10 \x01(\x05\x12\x13\n\x0b\x64\x65scription\x18\x14 \x01(\x0c\x12\x0b\n\x03url\x18\x15 \x01(\x0c\x12\x1c\n\x14\x66ree_asset_net_limit\x18\x16 \x01(\x03\x12#\n\x1bpublic_free_asset_net_limit\x18\x17 \x01(\x03\x12#\n\x1bpublic_free_asset_net_usage\x18\x18 \x01(\x03\x12#\n\x1bpublic_latest_free_net_time\x18\x19 \x01(\x03\x1a:\n\x0c\x46rozenSupply\x12\x15\n\rfrozen_amount\x18\x01 \x01(\x03\x12\x13\n\x0b\x66rozen_days\x18\x02 \x01(\x03\"f\n\x15TransferAssetContract\x12\x12\n\nasset_name\x18\x01 \x01(\x0c\x12\x15\n\rowner_address\x18\x02 \x01(\x0c\x12\x12\n\nto_address\x18\x03 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x04 \x01(\x03\".\n\x15UnfreezeAssetContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\"{\n\x13UpdateAssetContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\x0c\x12\x0b\n\x03url\x18\x03 \x01(\x0c\x12\x11\n\tnew_limit\x18\x04 \x01(\x03\x12\x18\n\x10new_public_limit\x18\x05 \x01(\x03\"n\n\x1dParticipateAssetIssueContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x12\n\nto_address\x18\x02 \x01(\x0c\x12\x12\n\nasset_name\x18\x03 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x04 \x01(\x03\"\xa3\x01\n\x15\x46reezeBalanceContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x16\n\x0e\x66rozen_balance\x18\x02 \x01(\x03\x12\x17\n\x0f\x66rozen_duration\x18\x03 \x01(\x03\x12(\n\x08resource\x18\n \x01(\x0e\x32\x16.protocol.ResourceCode\x12\x18\n\x10receiver_address\x18\x0f \x01(\x0c\"t\n\x17UnfreezeBalanceContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12(\n\x08resource\x18\n \x01(\x0e\x32\x16.protocol.ResourceCode\x12\x18\n\x10receiver_address\x18\x0f \x01(\x0c\"0\n\x17WithdrawBalanceContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\"M\n\x10TransferContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x12\n\nto_address\x18\x02 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x03\"\xad\x01\n\x17MarketSellAssetContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x15\n\rsell_token_id\x18\x02 \x01(\x0c\x12\x1b\n\x13sell_token_quantity\x18\x03 \x01(\x03\x12\x14\n\x0c\x62uy_token_id\x18\x04 \x01(\x0c\x12\x1a\n\x12\x62uy_token_quantity\x18\x05 \x01(\x03\x12\x15\n\rpre_price_key\x18\x06 \x01(\x0c\"D\n\x19MarketCancelOrderContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x10\n\x08order_id\x18\x02 \x01(\x0c\"^\n\x17ProposalApproveContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x13\n\x0bproposal_id\x18\x02 \x01(\x03\x12\x17\n\x0fis_add_approval\x18\x03 \x01(\x08\"\xa8\x01\n\x16ProposalCreateContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x44\n\nparameters\x18\x02 \x03(\x0b\x32\x30.protocol.ProposalCreateContract.ParametersEntry\x1a\x31\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"D\n\x16ProposalDeleteContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x13\n\x0bproposal_id\x18\x02 \x01(\x03\"\x87\x01\n\x13\x43reateSmartContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12-\n\x0cnew_contract\x18\x02 \x01(\x0b\x32\x17.protocol.SmartContract\x12\x18\n\x10\x63\x61ll_token_value\x18\x03 \x01(\x03\x12\x10\n\x08token_id\x18\x04 \x01(\x03\"\x95\x01\n\x14TriggerSmartContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x18\n\x10\x63ontract_address\x18\x02 \x01(\x0c\x12\x12\n\ncall_value\x18\x03 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\x12\x18\n\x10\x63\x61ll_token_value\x18\x05 \x01(\x03\x12\x10\n\x08token_id\x18\x06 \x01(\x03\"C\n\x10\x43learABIContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x18\n\x10\x63ontract_address\x18\x02 \x01(\x0c\"o\n\x15UpdateSettingContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x18\n\x10\x63ontract_address\x18\x02 \x01(\x0c\x12%\n\x1d\x63onsume_user_resource_percent\x18\x03 \x01(\x03\"i\n\x19UpdateEnergyLimitContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x18\n\x10\x63ontract_address\x18\x02 \x01(\x0c\x12\x1b\n\x13origin_energy_limit\x18\x03 \x01(\x03\"C\n\x17UpdateBrokerageContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x11\n\tbrokerage\x18\x02 \x01(\x05\"`\n\x11VoteAssetContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x14\n\x0cvote_address\x18\x02 \x03(\x0c\x12\x0f\n\x07support\x18\x03 \x01(\x08\x12\r\n\x05\x63ount\x18\x05 \x01(\x05\";\n\x15WitnessCreateContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x0b\n\x03url\x18\x02 \x01(\x0c\"B\n\x15WitnessUpdateContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x12\n\nupdate_url\x18\x0c \x01(\x0c\"\\\n\x13VoteWitnessContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x1d\n\x05votes\x18\x02 \x03(\x0b\x32\x0e.protocol.Vote\x12\x0f\n\x07support\x18\x03 \x01(\x08\"\x9b\x01\n\x16\x45xchangeCreateContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x16\n\x0e\x66irst_token_id\x18\x02 \x01(\x0c\x12\x1b\n\x13\x66irst_token_balance\x18\x03 \x01(\x03\x12\x17\n\x0fsecond_token_id\x18\x04 \x01(\x0c\x12\x1c\n\x14second_token_balance\x18\x05 \x01(\x03\"e\n\x16\x45xchangeInjectContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x13\n\x0b\x65xchange_id\x18\x02 \x01(\x03\x12\x10\n\x08token_id\x18\x03 \x01(\x0c\x12\r\n\x05quant\x18\x04 \x01(\x03\"g\n\x18\x45xchangeWithdrawContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x13\n\x0b\x65xchange_id\x18\x02 \x01(\x03\x12\x10\n\x08token_id\x18\x03 \x01(\x0c\x12\r\n\x05quant\x18\x04 \x01(\x03\"|\n\x1b\x45xchangeTransactionContract\x12\x15\n\rowner_address\x18\x01 \x01(\x0c\x12\x13\n\x0b\x65xchange_id\x18\x02 \x01(\x03\x12\x10\n\x08token_id\x18\x03 \x01(\x0c\x12\r\n\x05quant\x18\x04 \x01(\x03\x12\x10\n\x08\x65xpected\x18\x05 \x01(\x03\x62\x06proto3')



_ACCOUNTCREATECONTRACT = DESCRIPTOR.message_types_by_name['AccountCreateContract']
_ACCOUNTUPDATECONTRACT = DESCRIPTOR.message_types_by_name['AccountUpdateContract']
_SETACCOUNTIDCONTRACT = DESCRIPTOR.message_types_by_name['SetAccountIdContract']
_ACCOUNTPERMISSIONUPDATECONTRACT = DESCRIPTOR.message_types_by_name['AccountPermissionUpdateContract']
_ASSETISSUECONTRACT = DESCRIPTOR.message_types_by_name['AssetIssueContract']
_ASSETISSUECONTRACT_FROZENSUPPLY = _ASSETISSUECONTRACT.nested_types_by_name['FrozenSupply']
_TRANSFERASSETCONTRACT = DESCRIPTOR.message_types_by_name['TransferAssetContract']
_UNFREEZEASSETCONTRACT = DESCRIPTOR.message_types_by_name['UnfreezeAssetContract']
_UPDATEASSETCONTRACT = DESCRIPTOR.message_types_by_name['UpdateAssetContract']
_PARTICIPATEASSETISSUECONTRACT = DESCRIPTOR.message_types_by_name['ParticipateAssetIssueContract']
_FREEZEBALANCECONTRACT = DESCRIPTOR.message_types_by_name['FreezeBalanceContract']
_UNFREEZEBALANCECONTRACT = DESCRIPTOR.message_types_by_name['UnfreezeBalanceContract']
_WITHDRAWBALANCECONTRACT = DESCRIPTOR.message_types_by_name['WithdrawBalanceContract']
_TRANSFERCONTRACT = DESCRIPTOR.message_types_by_name['TransferContract']
_MARKETSELLASSETCONTRACT = DESCRIPTOR.message_types_by_name['MarketSellAssetContract']
_MARKETCANCELORDERCONTRACT = DESCRIPTOR.message_types_by_name['MarketCancelOrderContract']
_PROPOSALAPPROVECONTRACT = DESCRIPTOR.message_types_by_name['ProposalApproveContract']
_PROPOSALCREATECONTRACT = DESCRIPTOR.message_types_by_name['ProposalCreateContract']
_PROPOSALCREATECONTRACT_PARAMETERSENTRY = _PROPOSALCREATECONTRACT.nested_types_by_name['ParametersEntry']
_PROPOSALDELETECONTRACT = DESCRIPTOR.message_types_by_name['ProposalDeleteContract']
_CREATESMARTCONTRACT = DESCRIPTOR.message_types_by_name['CreateSmartContract']
_TRIGGERSMARTCONTRACT = DESCRIPTOR.message_types_by_name['TriggerSmartContract']
_CLEARABICONTRACT = DESCRIPTOR.message_types_by_name['ClearABIContract']
_UPDATESETTINGCONTRACT = DESCRIPTOR.message_types_by_name['UpdateSettingContract']
_UPDATEENERGYLIMITCONTRACT = DESCRIPTOR.message_types_by_name['UpdateEnergyLimitContract']
_UPDATEBROKERAGECONTRACT = DESCRIPTOR.message_types_by_name['UpdateBrokerageContract']
_VOTEASSETCONTRACT = DESCRIPTOR.message_types_by_name['VoteAssetContract']
_WITNESSCREATECONTRACT = DESCRIPTOR.message_types_by_name['WitnessCreateContract']
_WITNESSUPDATECONTRACT = DESCRIPTOR.message_types_by_name['WitnessUpdateContract']
_VOTEWITNESSCONTRACT = DESCRIPTOR.message_types_by_name['VoteWitnessContract']
_EXCHANGECREATECONTRACT = DESCRIPTOR.message_types_by_name['ExchangeCreateContract']
_EXCHANGEINJECTCONTRACT = DESCRIPTOR.message_types_by_name['ExchangeInjectContract']
_EXCHANGEWITHDRAWCONTRACT = DESCRIPTOR.message_types_by_name['ExchangeWithdrawContract']
_EXCHANGETRANSACTIONCONTRACT = DESCRIPTOR.message_types_by_name['ExchangeTransactionContract']
AccountCreateContract = _reflection.GeneratedProtocolMessageType('AccountCreateContract', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTCREATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.AccountCreateContract)
  })
_sym_db.RegisterMessage(AccountCreateContract)

AccountUpdateContract = _reflection.GeneratedProtocolMessageType('AccountUpdateContract', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTUPDATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.AccountUpdateContract)
  })
_sym_db.RegisterMessage(AccountUpdateContract)

SetAccountIdContract = _reflection.GeneratedProtocolMessageType('SetAccountIdContract', (_message.Message,), {
  'DESCRIPTOR' : _SETACCOUNTIDCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.SetAccountIdContract)
  })
_sym_db.RegisterMessage(SetAccountIdContract)

AccountPermissionUpdateContract = _reflection.GeneratedProtocolMessageType('AccountPermissionUpdateContract', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTPERMISSIONUPDATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.AccountPermissionUpdateContract)
  })
_sym_db.RegisterMessage(AccountPermissionUpdateContract)

AssetIssueContract = _reflection.GeneratedProtocolMessageType('AssetIssueContract', (_message.Message,), {

  'FrozenSupply' : _reflection.GeneratedProtocolMessageType('FrozenSupply', (_message.Message,), {
    'DESCRIPTOR' : _ASSETISSUECONTRACT_FROZENSUPPLY,
    '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
    # @@protoc_insertion_point(class_scope:protocol.AssetIssueContract.FrozenSupply)
    })
  ,
  'DESCRIPTOR' : _ASSETISSUECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.AssetIssueContract)
  })
_sym_db.RegisterMessage(AssetIssueContract)
_sym_db.RegisterMessage(AssetIssueContract.FrozenSupply)

TransferAssetContract = _reflection.GeneratedProtocolMessageType('TransferAssetContract', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFERASSETCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.TransferAssetContract)
  })
_sym_db.RegisterMessage(TransferAssetContract)

UnfreezeAssetContract = _reflection.GeneratedProtocolMessageType('UnfreezeAssetContract', (_message.Message,), {
  'DESCRIPTOR' : _UNFREEZEASSETCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.UnfreezeAssetContract)
  })
_sym_db.RegisterMessage(UnfreezeAssetContract)

UpdateAssetContract = _reflection.GeneratedProtocolMessageType('UpdateAssetContract', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEASSETCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.UpdateAssetContract)
  })
_sym_db.RegisterMessage(UpdateAssetContract)

ParticipateAssetIssueContract = _reflection.GeneratedProtocolMessageType('ParticipateAssetIssueContract', (_message.Message,), {
  'DESCRIPTOR' : _PARTICIPATEASSETISSUECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ParticipateAssetIssueContract)
  })
_sym_db.RegisterMessage(ParticipateAssetIssueContract)

FreezeBalanceContract = _reflection.GeneratedProtocolMessageType('FreezeBalanceContract', (_message.Message,), {
  'DESCRIPTOR' : _FREEZEBALANCECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.FreezeBalanceContract)
  })
_sym_db.RegisterMessage(FreezeBalanceContract)

UnfreezeBalanceContract = _reflection.GeneratedProtocolMessageType('UnfreezeBalanceContract', (_message.Message,), {
  'DESCRIPTOR' : _UNFREEZEBALANCECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.UnfreezeBalanceContract)
  })
_sym_db.RegisterMessage(UnfreezeBalanceContract)

WithdrawBalanceContract = _reflection.GeneratedProtocolMessageType('WithdrawBalanceContract', (_message.Message,), {
  'DESCRIPTOR' : _WITHDRAWBALANCECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.WithdrawBalanceContract)
  })
_sym_db.RegisterMessage(WithdrawBalanceContract)

TransferContract = _reflection.GeneratedProtocolMessageType('TransferContract', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFERCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.TransferContract)
  })
_sym_db.RegisterMessage(TransferContract)

MarketSellAssetContract = _reflection.GeneratedProtocolMessageType('MarketSellAssetContract', (_message.Message,), {
  'DESCRIPTOR' : _MARKETSELLASSETCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.MarketSellAssetContract)
  })
_sym_db.RegisterMessage(MarketSellAssetContract)

MarketCancelOrderContract = _reflection.GeneratedProtocolMessageType('MarketCancelOrderContract', (_message.Message,), {
  'DESCRIPTOR' : _MARKETCANCELORDERCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.MarketCancelOrderContract)
  })
_sym_db.RegisterMessage(MarketCancelOrderContract)

ProposalApproveContract = _reflection.GeneratedProtocolMessageType('ProposalApproveContract', (_message.Message,), {
  'DESCRIPTOR' : _PROPOSALAPPROVECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ProposalApproveContract)
  })
_sym_db.RegisterMessage(ProposalApproveContract)

ProposalCreateContract = _reflection.GeneratedProtocolMessageType('ProposalCreateContract', (_message.Message,), {

  'ParametersEntry' : _reflection.GeneratedProtocolMessageType('ParametersEntry', (_message.Message,), {
    'DESCRIPTOR' : _PROPOSALCREATECONTRACT_PARAMETERSENTRY,
    '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
    # @@protoc_insertion_point(class_scope:protocol.ProposalCreateContract.ParametersEntry)
    })
  ,
  'DESCRIPTOR' : _PROPOSALCREATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ProposalCreateContract)
  })
_sym_db.RegisterMessage(ProposalCreateContract)
_sym_db.RegisterMessage(ProposalCreateContract.ParametersEntry)

ProposalDeleteContract = _reflection.GeneratedProtocolMessageType('ProposalDeleteContract', (_message.Message,), {
  'DESCRIPTOR' : _PROPOSALDELETECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ProposalDeleteContract)
  })
_sym_db.RegisterMessage(ProposalDeleteContract)

CreateSmartContract = _reflection.GeneratedProtocolMessageType('CreateSmartContract', (_message.Message,), {
  'DESCRIPTOR' : _CREATESMARTCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.CreateSmartContract)
  })
_sym_db.RegisterMessage(CreateSmartContract)

TriggerSmartContract = _reflection.GeneratedProtocolMessageType('TriggerSmartContract', (_message.Message,), {
  'DESCRIPTOR' : _TRIGGERSMARTCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.TriggerSmartContract)
  })
_sym_db.RegisterMessage(TriggerSmartContract)

ClearABIContract = _reflection.GeneratedProtocolMessageType('ClearABIContract', (_message.Message,), {
  'DESCRIPTOR' : _CLEARABICONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ClearABIContract)
  })
_sym_db.RegisterMessage(ClearABIContract)

UpdateSettingContract = _reflection.GeneratedProtocolMessageType('UpdateSettingContract', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESETTINGCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.UpdateSettingContract)
  })
_sym_db.RegisterMessage(UpdateSettingContract)

UpdateEnergyLimitContract = _reflection.GeneratedProtocolMessageType('UpdateEnergyLimitContract', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEENERGYLIMITCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.UpdateEnergyLimitContract)
  })
_sym_db.RegisterMessage(UpdateEnergyLimitContract)

UpdateBrokerageContract = _reflection.GeneratedProtocolMessageType('UpdateBrokerageContract', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEBROKERAGECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.UpdateBrokerageContract)
  })
_sym_db.RegisterMessage(UpdateBrokerageContract)

VoteAssetContract = _reflection.GeneratedProtocolMessageType('VoteAssetContract', (_message.Message,), {
  'DESCRIPTOR' : _VOTEASSETCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.VoteAssetContract)
  })
_sym_db.RegisterMessage(VoteAssetContract)

WitnessCreateContract = _reflection.GeneratedProtocolMessageType('WitnessCreateContract', (_message.Message,), {
  'DESCRIPTOR' : _WITNESSCREATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.WitnessCreateContract)
  })
_sym_db.RegisterMessage(WitnessCreateContract)

WitnessUpdateContract = _reflection.GeneratedProtocolMessageType('WitnessUpdateContract', (_message.Message,), {
  'DESCRIPTOR' : _WITNESSUPDATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.WitnessUpdateContract)
  })
_sym_db.RegisterMessage(WitnessUpdateContract)

VoteWitnessContract = _reflection.GeneratedProtocolMessageType('VoteWitnessContract', (_message.Message,), {
  'DESCRIPTOR' : _VOTEWITNESSCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.VoteWitnessContract)
  })
_sym_db.RegisterMessage(VoteWitnessContract)

ExchangeCreateContract = _reflection.GeneratedProtocolMessageType('ExchangeCreateContract', (_message.Message,), {
  'DESCRIPTOR' : _EXCHANGECREATECONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ExchangeCreateContract)
  })
_sym_db.RegisterMessage(ExchangeCreateContract)

ExchangeInjectContract = _reflection.GeneratedProtocolMessageType('ExchangeInjectContract', (_message.Message,), {
  'DESCRIPTOR' : _EXCHANGEINJECTCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ExchangeInjectContract)
  })
_sym_db.RegisterMessage(ExchangeInjectContract)

ExchangeWithdrawContract = _reflection.GeneratedProtocolMessageType('ExchangeWithdrawContract', (_message.Message,), {
  'DESCRIPTOR' : _EXCHANGEWITHDRAWCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ExchangeWithdrawContract)
  })
_sym_db.RegisterMessage(ExchangeWithdrawContract)

ExchangeTransactionContract = _reflection.GeneratedProtocolMessageType('ExchangeTransactionContract', (_message.Message,), {
  'DESCRIPTOR' : _EXCHANGETRANSACTIONCONTRACT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.core.contract_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ExchangeTransactionContract)
  })
_sym_db.RegisterMessage(ExchangeTransactionContract)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PROPOSALCREATECONTRACT_PARAMETERSENTRY._options = None
  _PROPOSALCREATECONTRACT_PARAMETERSENTRY._serialized_options = b'8\001'
  _ACCOUNTCREATECONTRACT._serialized_start=114
  _ACCOUNTCREATECONTRACT._serialized_end=222
  _ACCOUNTUPDATECONTRACT._serialized_start=224
  _ACCOUNTUPDATECONTRACT._serialized_end=292
  _SETACCOUNTIDCONTRACT._serialized_start=294
  _SETACCOUNTIDCONTRACT._serialized_end=359
  _ACCOUNTPERMISSIONUPDATECONTRACT._serialized_start=362
  _ACCOUNTPERMISSIONUPDATECONTRACT._serialized_end=533
  _ASSETISSUECONTRACT._serialized_start=536
  _ASSETISSUECONTRACT._serialized_end=1064
  _ASSETISSUECONTRACT_FROZENSUPPLY._serialized_start=1006
  _ASSETISSUECONTRACT_FROZENSUPPLY._serialized_end=1064
  _TRANSFERASSETCONTRACT._serialized_start=1066
  _TRANSFERASSETCONTRACT._serialized_end=1168
  _UNFREEZEASSETCONTRACT._serialized_start=1170
  _UNFREEZEASSETCONTRACT._serialized_end=1216
  _UPDATEASSETCONTRACT._serialized_start=1218
  _UPDATEASSETCONTRACT._serialized_end=1341
  _PARTICIPATEASSETISSUECONTRACT._serialized_start=1343
  _PARTICIPATEASSETISSUECONTRACT._serialized_end=1453
  _FREEZEBALANCECONTRACT._serialized_start=1456
  _FREEZEBALANCECONTRACT._serialized_end=1619
  _UNFREEZEBALANCECONTRACT._serialized_start=1621
  _UNFREEZEBALANCECONTRACT._serialized_end=1737
  _WITHDRAWBALANCECONTRACT._serialized_start=1739
  _WITHDRAWBALANCECONTRACT._serialized_end=1787
  _TRANSFERCONTRACT._serialized_start=1789
  _TRANSFERCONTRACT._serialized_end=1866
  _MARKETSELLASSETCONTRACT._serialized_start=1869
  _MARKETSELLASSETCONTRACT._serialized_end=2042
  _MARKETCANCELORDERCONTRACT._serialized_start=2044
  _MARKETCANCELORDERCONTRACT._serialized_end=2112
  _PROPOSALAPPROVECONTRACT._serialized_start=2114
  _PROPOSALAPPROVECONTRACT._serialized_end=2208
  _PROPOSALCREATECONTRACT._serialized_start=2211
  _PROPOSALCREATECONTRACT._serialized_end=2379
  _PROPOSALCREATECONTRACT_PARAMETERSENTRY._serialized_start=2330
  _PROPOSALCREATECONTRACT_PARAMETERSENTRY._serialized_end=2379
  _PROPOSALDELETECONTRACT._serialized_start=2381
  _PROPOSALDELETECONTRACT._serialized_end=2449
  _CREATESMARTCONTRACT._serialized_start=2452
  _CREATESMARTCONTRACT._serialized_end=2587
  _TRIGGERSMARTCONTRACT._serialized_start=2590
  _TRIGGERSMARTCONTRACT._serialized_end=2739
  _CLEARABICONTRACT._serialized_start=2741
  _CLEARABICONTRACT._serialized_end=2808
  _UPDATESETTINGCONTRACT._serialized_start=2810
  _UPDATESETTINGCONTRACT._serialized_end=2921
  _UPDATEENERGYLIMITCONTRACT._serialized_start=2923
  _UPDATEENERGYLIMITCONTRACT._serialized_end=3028
  _UPDATEBROKERAGECONTRACT._serialized_start=3030
  _UPDATEBROKERAGECONTRACT._serialized_end=3097
  _VOTEASSETCONTRACT._serialized_start=3099
  _VOTEASSETCONTRACT._serialized_end=3195
  _WITNESSCREATECONTRACT._serialized_start=3197
  _WITNESSCREATECONTRACT._serialized_end=3256
  _WITNESSUPDATECONTRACT._serialized_start=3258
  _WITNESSUPDATECONTRACT._serialized_end=3324
  _VOTEWITNESSCONTRACT._serialized_start=3326
  _VOTEWITNESSCONTRACT._serialized_end=3418
  _EXCHANGECREATECONTRACT._serialized_start=3421
  _EXCHANGECREATECONTRACT._serialized_end=3576
  _EXCHANGEINJECTCONTRACT._serialized_start=3578
  _EXCHANGEINJECTCONTRACT._serialized_end=3679
  _EXCHANGEWITHDRAWCONTRACT._serialized_start=3681
  _EXCHANGEWITHDRAWCONTRACT._serialized_end=3784
  _EXCHANGETRANSACTIONCONTRACT._serialized_start=3786
  _EXCHANGETRANSACTIONCONTRACT._serialized_end=3910
# @@protoc_insertion_point(module_scope)
