# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: graphsenselib/ingest/tron/grpc/api/tron_api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from graphsenselib.ingest.tron.grpc.core import common_pb2 as graphsenselib_dot_ingest_dot_tron_dot_grpc_dot_core_dot_common__pb2
from graphsenselib.ingest.tron.grpc.core import chain_pb2 as graphsenselib_dot_ingest_dot_tron_dot_grpc_dot_core_dot_chain__pb2
from graphsenselib.ingest.tron.grpc.core import contract_pb2 as graphsenselib_dot_ingest_dot_tron_dot_grpc_dot_core_dot_contract__pb2
from graphsenselib.ingest.tron.grpc.core import response_pb2 as graphsenselib_dot_ingest_dot_tron_dot_grpc_dot_core_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1graphsenselib/ingest/tron/grpc/api/tron_api.proto\x12\x08protocol\x1a\x30graphsenselib/ingest/tron/grpc/core/common.proto\x1a/graphsenselib/ingest/tron/grpc/core/chain.proto\x1a\x32graphsenselib/ingest/tron/grpc/core/contract.proto\x1a\x32graphsenselib/ingest/tron/grpc/core/response.proto\"\x0e\n\x0c\x45mptyMessage\"\x1c\n\rNumberMessage\x12\x0b\n\x03num\x18\x01 \x01(\x03\"\x1d\n\x0c\x42ytesMessage\x12\r\n\x05value\x18\x01 \x01(\x0c\".\n\nBlockLimit\x12\x10\n\x08startNum\x18\x01 \x01(\x03\x12\x0e\n\x06\x65ndNum\x18\x02 \x01(\x03\"(\n\x15\x41\x63\x63ountAddressMessage\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\x0c\"\x1e\n\x10\x41\x63\x63ountIdMessage\x12\n\n\x02id\x18\x17 \x01(\x0c\"1\n\x10PaginatedMessage\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\r\n\x05limit\x18\x02 \x01(\x03\"L\n\x13\x45\x61syTransferMessage\x12\x12\n\npassPhrase\x18\x01 \x01(\x0c\x12\x11\n\ttoAddress\x18\x02 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x03\"b\n\x18\x45\x61syTransferAssetMessage\x12\x12\n\npassPhrase\x18\x01 \x01(\x0c\x12\x11\n\ttoAddress\x18\x02 \x01(\x0c\x12\x0f\n\x07\x61ssetId\x18\x03 \x01(\t\x12\x0e\n\x06\x61mount\x18\x04 \x01(\x03\"U\n\x1c\x45\x61syTransferByPrivateMessage\x12\x12\n\nprivateKey\x18\x01 \x01(\x0c\x12\x11\n\ttoAddress\x18\x02 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x03\"k\n!EasyTransferAssetByPrivateMessage\x12\x12\n\nprivateKey\x18\x01 \x01(\x0c\x12\x11\n\ttoAddress\x18\x02 \x01(\x0c\x12\x0f\n\x07\x61ssetId\x18\x03 \x01(\t\x12\x0e\n\x06\x61mount\x18\x04 \x01(\x03\"+\n\x11ViewingKeyMessage\x12\n\n\x02\x61k\x18\x01 \x01(\x0c\x12\n\n\x02nk\x18\x02 \x01(\x0c\"\x9e\x01\n\x13ShieldedAddressInfo\x12\n\n\x02sk\x18\x01 \x01(\x0c\x12\x0b\n\x03\x61sk\x18\x02 \x01(\x0c\x12\x0b\n\x03nsk\x18\x03 \x01(\x0c\x12\x0b\n\x03ovk\x18\x04 \x01(\x0c\x12\n\n\x02\x61k\x18\x05 \x01(\x0c\x12\n\n\x02nk\x18\x06 \x01(\x0c\x12\x0b\n\x03ivk\x18\x07 \x01(\x0c\x12\t\n\x01\x64\x18\x08 \x01(\x0c\x12\x0b\n\x03pkD\x18\t \x01(\x0c\x12\x17\n\x0fpayment_address\x18\n \x01(\t\"f\n\x15PaymentAddressMessage\x12\'\n\x01\x64\x18\x01 \x01(\x0b\x32\x1c.protocol.DiversifierMessage\x12\x0b\n\x03pkD\x18\x02 \x01(\x0c\x12\x17\n\x0fpayment_address\x18\x03 \x01(\t\"\x1f\n\x12\x44iversifierMessage\x12\t\n\x01\x64\x18\x01 \x01(\x0c\"C\n\x1a\x45xpandedSpendingKeyMessage\x12\x0b\n\x03\x61sk\x18\x01 \x01(\x0c\x12\x0b\n\x03nsk\x18\x02 \x01(\x0c\x12\x0b\n\x03ovk\x18\x03 \x01(\x0c\"(\n\x19IncomingViewingKeyMessage\x12\x0b\n\x03ivk\x18\x01 \x01(\x0c\"\x81\x01\n$IncomingViewingKeyDiversifierMessage\x12\x30\n\x03ivk\x18\x01 \x01(\x0b\x32#.protocol.IncomingViewingKeyMessage\x12\'\n\x01\x64\x18\x02 \x01(\x0b\x32\x1c.protocol.DiversifierMessage\"+\n\x0bReceiveNote\x12\x1c\n\x04note\x18\x01 \x01(\x0b\x32\x0e.protocol.Note\"f\n\x0eSpendNoteTRC20\x12\x1c\n\x04note\x18\x01 \x01(\x0b\x32\x0e.protocol.Note\x12\r\n\x05\x61lpha\x18\x02 \x01(\x0c\x12\x0c\n\x04root\x18\x03 \x01(\x0c\x12\x0c\n\x04path\x18\x04 \x01(\x0c\x12\x0b\n\x03pos\x18\x05 \x01(\x03\"\x9d\x02\n\x1ePrivateShieldedTRC20Parameters\x12\x0b\n\x03\x61sk\x18\x01 \x01(\x0c\x12\x0b\n\x03nsk\x18\x02 \x01(\x0c\x12\x0b\n\x03ovk\x18\x03 \x01(\x0c\x12\x13\n\x0b\x66rom_amount\x18\x04 \x01(\t\x12\x31\n\x0fshielded_spends\x18\x05 \x03(\x0b\x32\x18.protocol.SpendNoteTRC20\x12\x30\n\x11shielded_receives\x18\x06 \x03(\x0b\x32\x15.protocol.ReceiveNote\x12\x1e\n\x16transparent_to_address\x18\x07 \x01(\x0c\x12\x11\n\tto_amount\x18\x08 \x01(\t\x12\'\n\x1fshielded_TRC20_contract_address\x18\t \x01(\x0c\"\xa6\x02\n(PrivateShieldedTRC20ParametersWithoutAsk\x12\n\n\x02\x61k\x18\x01 \x01(\x0c\x12\x0b\n\x03nsk\x18\x02 \x01(\x0c\x12\x0b\n\x03ovk\x18\x03 \x01(\x0c\x12\x13\n\x0b\x66rom_amount\x18\x04 \x01(\t\x12\x31\n\x0fshielded_spends\x18\x05 \x03(\x0b\x32\x18.protocol.SpendNoteTRC20\x12\x30\n\x11shielded_receives\x18\x06 \x03(\x0b\x32\x15.protocol.ReceiveNote\x12\x1e\n\x16transparent_to_address\x18\x07 \x01(\x0c\x12\x11\n\tto_amount\x18\x08 \x01(\t\x12\'\n\x1fshielded_TRC20_contract_address\x18\t \x01(\x0c\"\x8f\x01\n\x10SpendDescription\x12\x18\n\x10value_commitment\x18\x01 \x01(\x0c\x12\x0e\n\x06\x61nchor\x18\x02 \x01(\x0c\x12\x11\n\tnullifier\x18\x03 \x01(\x0c\x12\n\n\x02rk\x18\x04 \x01(\x0c\x12\x0f\n\x07zkproof\x18\x05 \x01(\x0c\x12!\n\x19spend_authority_signature\x18\x06 \x01(\x0c\"\x83\x01\n\x12ReceiveDescription\x12\x18\n\x10value_commitment\x18\x01 \x01(\x0c\x12\x17\n\x0fnote_commitment\x18\x02 \x01(\x0c\x12\x0b\n\x03\x65pk\x18\x03 \x01(\x0c\x12\r\n\x05\x63_enc\x18\x04 \x01(\x0c\x12\r\n\x05\x63_out\x18\x05 \x01(\x0c\x12\x0f\n\x07zkproof\x18\x06 \x01(\x0c\"\xf4\x01\n\x17ShieldedTRC20Parameters\x12\x35\n\x11spend_description\x18\x01 \x03(\x0b\x32\x1a.protocol.SpendDescription\x12\x39\n\x13receive_description\x18\x02 \x03(\x0b\x32\x1c.protocol.ReceiveDescription\x12\x19\n\x11\x62inding_signature\x18\x03 \x01(\x0c\x12\x14\n\x0cmessage_hash\x18\x04 \x01(\x0c\x12\x1e\n\x16trigger_contract_input\x18\x05 \x01(\t\x12\x16\n\x0eparameter_type\x18\x06 \x01(\t\"\xad\x01\n\x19IvkDecryptTRC20Parameters\x12\x19\n\x11start_block_index\x18\x01 \x01(\x03\x12\x17\n\x0f\x65nd_block_index\x18\x02 \x01(\x03\x12\'\n\x1fshielded_TRC20_contract_address\x18\x03 \x01(\x0c\x12\x0b\n\x03ivk\x18\x04 \x01(\x0c\x12\n\n\x02\x61k\x18\x05 \x01(\x0c\x12\n\n\x02nk\x18\x06 \x01(\x0c\x12\x0e\n\x06\x65vents\x18\x07 \x03(\t\"\x95\x01\n\x19OvkDecryptTRC20Parameters\x12\x19\n\x11start_block_index\x18\x01 \x01(\x03\x12\x17\n\x0f\x65nd_block_index\x18\x02 \x01(\x03\x12\x0b\n\x03ovk\x18\x03 \x01(\x0c\x12\'\n\x1fshielded_TRC20_contract_address\x18\x04 \x01(\x0c\x12\x0e\n\x06\x65vents\x18\x05 \x03(\t\"\x84\x01\n\x11NfTRC20Parameters\x12\x1c\n\x04note\x18\x01 \x01(\x0b\x32\x0e.protocol.Note\x12\n\n\x02\x61k\x18\x02 \x01(\x0c\x12\n\n\x02nk\x18\x03 \x01(\x0c\x12\x10\n\x08position\x18\x04 \x01(\x03\x12\'\n\x1fshielded_TRC20_contract_address\x18\x05 \x01(\x0c\"\xd9\x01\n&ShieldedTRC20TriggerContractParameters\x12\x44\n\x19shielded_TRC20_Parameters\x18\x01 \x01(\x0b\x32!.protocol.ShieldedTRC20Parameters\x12\x39\n\x19spend_authority_signature\x18\x02 \x03(\x0b\x32\x16.protocol.BytesMessage\x12\x0e\n\x06\x61mount\x18\x03 \x01(\t\x12\x1e\n\x16transparent_to_address\x18\x04 \x01(\x0c\x32\xb3L\n\x06Wallet\x12L\n\x14\x42roadcastTransaction\x12\x15.protocol.Transaction\x1a\x1b.protocol.TransactionReturn\"\x00\x12R\n\x17\x43reateCommonTransaction\x12\x15.protocol.Transaction\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rCreateAccount\x12\x1f.protocol.AccountCreateContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0e\x43reateAccount2\x12\x1f.protocol.AccountCreateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rUpdateAccount\x12\x1f.protocol.AccountUpdateContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0eUpdateAccount2\x12\x1f.protocol.AccountUpdateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12G\n\x0cSetAccountId\x12\x1e.protocol.SetAccountIdContract\x1a\x15.protocol.Transaction\"\x00\x12\x66\n\x17\x41\x63\x63ountPermissionUpdate\x12).protocol.AccountPermissionUpdateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12H\n\x11\x43reateTransaction\x12\x1a.protocol.TransferContract\x1a\x15.protocol.Transaction\"\x00\x12R\n\x12\x43reateTransaction2\x12\x1a.protocol.TransferContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\x10\x43reateAssetIssue\x12\x1c.protocol.AssetIssueContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x11\x43reateAssetIssue2\x12\x1c.protocol.AssetIssueContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12\x45\n\x0bUpdateAsset\x12\x1d.protocol.UpdateAssetContract\x1a\x15.protocol.Transaction\"\x00\x12O\n\x0cUpdateAsset2\x12\x1d.protocol.UpdateAssetContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rTransferAsset\x12\x1f.protocol.TransferAssetContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0eTransferAsset2\x12\x1f.protocol.TransferAssetContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12Y\n\x15ParticipateAssetIssue\x12\'.protocol.ParticipateAssetIssueContract\x1a\x15.protocol.Transaction\"\x00\x12\x63\n\x16ParticipateAssetIssue2\x12\'.protocol.ParticipateAssetIssueContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rUnfreezeAsset\x12\x1f.protocol.UnfreezeAssetContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0eUnfreezeAsset2\x12\x1f.protocol.UnfreezeAssetContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rCreateWitness\x12\x1f.protocol.WitnessCreateContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0e\x43reateWitness2\x12\x1f.protocol.WitnessCreateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rUpdateWitness\x12\x1f.protocol.WitnessUpdateContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0eUpdateWitness2\x12\x1f.protocol.WitnessUpdateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12V\n\x0fUpdateBrokerage\x12!.protocol.UpdateBrokerageContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12L\n\x12VoteWitnessAccount\x12\x1d.protocol.VoteWitnessContract\x1a\x15.protocol.Transaction\"\x00\x12V\n\x13VoteWitnessAccount2\x12\x1d.protocol.VoteWitnessContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12I\n\rFreezeBalance\x12\x1f.protocol.FreezeBalanceContract\x1a\x15.protocol.Transaction\"\x00\x12S\n\x0e\x46reezeBalance2\x12\x1f.protocol.FreezeBalanceContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12M\n\x0fUnfreezeBalance\x12!.protocol.UnfreezeBalanceContract\x1a\x15.protocol.Transaction\"\x00\x12W\n\x10UnfreezeBalance2\x12!.protocol.UnfreezeBalanceContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12M\n\x0fWithdrawBalance\x12!.protocol.WithdrawBalanceContract\x1a\x15.protocol.Transaction\"\x00\x12W\n\x10WithdrawBalance2\x12!.protocol.WithdrawBalanceContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12T\n\x0eProposalCreate\x12 .protocol.ProposalCreateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12V\n\x0fProposalApprove\x12!.protocol.ProposalApproveContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12T\n\x0eProposalDelete\x12 .protocol.ProposalDeleteContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12Q\n\x0e\x44\x65ployContract\x12\x1d.protocol.CreateSmartContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12R\n\rUpdateSetting\x12\x1f.protocol.UpdateSettingContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12Z\n\x11UpdateEnergyLimit\x12#.protocol.UpdateEnergyLimitContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12P\n\x10\x43learContractABI\x12\x1a.protocol.ClearABIContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12S\n\x0fTriggerContract\x12\x1e.protocol.TriggerSmartContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12[\n\x17TriggerConstantContract\x12\x1e.protocol.TriggerSmartContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12T\n\x0e\x45xchangeCreate\x12 .protocol.ExchangeCreateContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12T\n\x0e\x45xchangeInject\x12 .protocol.ExchangeInjectContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12X\n\x10\x45xchangeWithdraw\x12\".protocol.ExchangeWithdrawContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12^\n\x13\x45xchangeTransaction\x12%.protocol.ExchangeTransactionContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12V\n\x0fMarketSellAsset\x12!.protocol.MarketSellAssetContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12Z\n\x11MarketCancelOrder\x12#.protocol.MarketCancelOrderContract\x1a\x1e.protocol.TransactionExtention\"\x00\x12;\n\x0bGetNodeInfo\x12\x16.protocol.EmptyMessage\x1a\x12.protocol.NodeInfo\"\x00\x12\x39\n\tListNodes\x12\x16.protocol.EmptyMessage\x1a\x12.protocol.NodeList\"\x00\x12I\n\x12GetChainParameters\x12\x16.protocol.EmptyMessage\x1a\x19.protocol.ChainParameters\"\x00\x12\x45\n\x10TotalTransaction\x12\x16.protocol.EmptyMessage\x1a\x17.protocol.NumberMessage\"\x00\x12K\n\x16GetNextMaintenanceTime\x12\x16.protocol.EmptyMessage\x1a\x17.protocol.NumberMessage\"\x00\x12T\n\x18GetTransactionSignWeight\x12\x15.protocol.Transaction\x1a\x1f.protocol.TransactionSignWeight\"\x00\x12X\n\x1aGetTransactionApprovedList\x12\x15.protocol.Transaction\x1a!.protocol.TransactionApprovedList\"\x00\x12\x42\n\nGetAccount\x12\x1f.protocol.AccountAddressMessage\x1a\x11.protocol.Account\"\x00\x12\x41\n\x0eGetAccountById\x12\x1a.protocol.AccountIdMessage\x1a\x11.protocol.Account\"\x00\x12O\n\rGetAccountNet\x12\x1f.protocol.AccountAddressMessage\x1a\x1b.protocol.AccountNetMessage\"\x00\x12Y\n\x12GetAccountResource\x12\x1f.protocol.AccountAddressMessage\x1a .protocol.AccountResourceMessage\"\x00\x12U\n\x16GetAssetIssueByAccount\x12\x1f.protocol.AccountAddressMessage\x1a\x18.protocol.AssetIssueList\"\x00\x12M\n\x13GetAssetIssueByName\x12\x16.protocol.BytesMessage\x1a\x1c.protocol.AssetIssueContract\"\x00\x12M\n\x17GetAssetIssueListByName\x12\x16.protocol.BytesMessage\x1a\x18.protocol.AssetIssueList\"\x00\x12K\n\x11GetAssetIssueById\x12\x16.protocol.BytesMessage\x1a\x1c.protocol.AssetIssueContract\"\x00\x12G\n\x11GetAssetIssueList\x12\x16.protocol.EmptyMessage\x1a\x18.protocol.AssetIssueList\"\x00\x12T\n\x1aGetPaginatedAssetIssueList\x12\x1a.protocol.PaginatedMessage\x1a\x18.protocol.AssetIssueList\"\x00\x12\x38\n\x0bGetNowBlock\x12\x16.protocol.EmptyMessage\x1a\x0f.protocol.Block\"\x00\x12\x42\n\x0cGetNowBlock2\x12\x16.protocol.EmptyMessage\x1a\x18.protocol.BlockExtention\"\x00\x12;\n\rGetBlockByNum\x12\x17.protocol.NumberMessage\x1a\x0f.protocol.Block\"\x00\x12\x45\n\x0eGetBlockByNum2\x12\x17.protocol.NumberMessage\x1a\x18.protocol.BlockExtention\"\x00\x12\x39\n\x0cGetBlockById\x12\x16.protocol.BytesMessage\x1a\x0f.protocol.Block\"\x00\x12\x42\n\x13GetBlockByLimitNext\x12\x14.protocol.BlockLimit\x1a\x13.protocol.BlockList\"\x00\x12L\n\x14GetBlockByLimitNext2\x12\x14.protocol.BlockLimit\x1a\x1c.protocol.BlockListExtention\"\x00\x12\x45\n\x13GetBlockByLatestNum\x12\x17.protocol.NumberMessage\x1a\x13.protocol.BlockList\"\x00\x12O\n\x14GetBlockByLatestNum2\x12\x17.protocol.NumberMessage\x1a\x1c.protocol.BlockListExtention\"\x00\x12S\n\x1dGetTransactionCountByBlockNum\x12\x17.protocol.NumberMessage\x1a\x17.protocol.NumberMessage\"\x00\x12\x45\n\x12GetTransactionById\x12\x16.protocol.BytesMessage\x1a\x15.protocol.Transaction\"\x00\x12M\n\x16GetTransactionInfoById\x12\x16.protocol.BytesMessage\x1a\x19.protocol.TransactionInfo\"\x00\x12X\n\x1cGetTransactionInfoByBlockNum\x12\x17.protocol.NumberMessage\x1a\x1d.protocol.TransactionInfoList\"\x00\x12@\n\x0bGetContract\x12\x16.protocol.BytesMessage\x1a\x17.protocol.SmartContract\"\x00\x12O\n\x0fGetContractInfo\x12\x16.protocol.BytesMessage\x1a\".protocol.SmartContractDataWrapper\"\x00\x12@\n\rListWitnesses\x12\x16.protocol.EmptyMessage\x1a\x15.protocol.WitnessList\"\x00\x12\x45\n\x10GetBrokerageInfo\x12\x16.protocol.BytesMessage\x1a\x17.protocol.NumberMessage\"\x00\x12\x42\n\rGetRewardInfo\x12\x16.protocol.BytesMessage\x1a\x17.protocol.NumberMessage\"\x00\x12]\n\x14GetDelegatedResource\x12\".protocol.DelegatedResourceMessage\x1a\x1f.protocol.DelegatedResourceList\"\x00\x12\x65\n GetDelegatedResourceAccountIndex\x12\x16.protocol.BytesMessage\x1a\'.protocol.DelegatedResourceAccountIndex\"\x00\x12\x41\n\rListProposals\x12\x16.protocol.EmptyMessage\x1a\x16.protocol.ProposalList\"\x00\x12?\n\x0fGetProposalById\x12\x16.protocol.BytesMessage\x1a\x12.protocol.Proposal\"\x00\x12P\n\x18GetPaginatedProposalList\x12\x1a.protocol.PaginatedMessage\x1a\x16.protocol.ProposalList\"\x00\x12\x41\n\rListExchanges\x12\x16.protocol.EmptyMessage\x1a\x16.protocol.ExchangeList\"\x00\x12?\n\x0fGetExchangeById\x12\x16.protocol.BytesMessage\x1a\x12.protocol.Exchange\"\x00\x12P\n\x18GetPaginatedExchangeList\x12\x1a.protocol.PaginatedMessage\x1a\x16.protocol.ExchangeList\"\x00\x12\x61\n\x1bScanShieldedTRC20NotesByIvk\x12#.protocol.IvkDecryptTRC20Parameters\x1a\x1b.protocol.DecryptNotesTRC20\"\x00\x12\x61\n\x1bScanShieldedTRC20NotesByOvk\x12#.protocol.OvkDecryptTRC20Parameters\x1a\x1b.protocol.DecryptNotesTRC20\"\x00\x12\\\n IsShieldedTRC20ContractNoteSpent\x12\x1b.protocol.NfTRC20Parameters\x1a\x19.protocol.NullifierResult\"\x00\x12N\n\x17GetMarketOrderByAccount\x12\x16.protocol.BytesMessage\x1a\x19.protocol.MarketOrderList\"\x00\x12\x45\n\x12GetMarketOrderById\x12\x16.protocol.BytesMessage\x1a\x15.protocol.MarketOrder\"\x00\x12N\n\x14GetMarketPriceByPair\x12\x19.protocol.MarketOrderPair\x1a\x19.protocol.MarketPriceList\"\x00\x12R\n\x18GetMarketOrderListByPair\x12\x19.protocol.MarketOrderPair\x1a\x19.protocol.MarketOrderList\"\x00\x12L\n\x11GetMarketPairList\x12\x16.protocol.EmptyMessage\x1a\x1d.protocol.MarketOrderPairList\"\x00\x12H\n\x12GetTransactionSign\x12\x19.protocol.TransactionSign\x1a\x15.protocol.Transaction\"\x00\x12R\n\x13GetTransactionSign2\x12\x19.protocol.TransactionSign\x1a\x1e.protocol.TransactionExtention\"\x00\x12Y\n\x11\x45\x61syTransferAsset\x12\".protocol.EasyTransferAssetMessage\x1a\x1e.protocol.EasyTransferResponse\"\x00\x12k\n\x1a\x45\x61syTransferAssetByPrivate\x12+.protocol.EasyTransferAssetByPrivateMessage\x1a\x1e.protocol.EasyTransferResponse\"\x00\x12O\n\x0c\x45\x61syTransfer\x12\x1d.protocol.EasyTransferMessage\x1a\x1e.protocol.EasyTransferResponse\"\x00\x12\x61\n\x15\x45\x61syTransferByPrivate\x12&.protocol.EasyTransferByPrivateMessage\x1a\x1e.protocol.EasyTransferResponse\"\x00\x12\x41\n\rCreateAddress\x12\x16.protocol.BytesMessage\x1a\x16.protocol.BytesMessage\"\x00\x12N\n\x0fGenerateAddress\x12\x16.protocol.EmptyMessage\x1a!.protocol.AddressPrKeyPairMessage\"\x00\x12\x46\n\x07\x41\x64\x64Sign\x12\x19.protocol.TransactionSign\x1a\x1e.protocol.TransactionExtention\"\x00\x12\x42\n\x0eGetSpendingKey\x12\x16.protocol.EmptyMessage\x1a\x16.protocol.BytesMessage\"\x00\x12X\n\x16GetExpandedSpendingKey\x12\x16.protocol.BytesMessage\x1a$.protocol.ExpandedSpendingKeyMessage\"\x00\x12@\n\x0cGetAkFromAsk\x12\x16.protocol.BytesMessage\x1a\x16.protocol.BytesMessage\"\x00\x12@\n\x0cGetNkFromNsk\x12\x16.protocol.BytesMessage\x1a\x16.protocol.BytesMessage\"\x00\x12[\n\x15GetIncomingViewingKey\x12\x1b.protocol.ViewingKeyMessage\x1a#.protocol.IncomingViewingKeyMessage\"\x00\x12H\n\x0eGetDiversifier\x12\x16.protocol.EmptyMessage\x1a\x1c.protocol.DiversifierMessage\"\x00\x12i\n\x14GetZenPaymentAddress\x12..protocol.IncomingViewingKeyDiversifierMessage\x1a\x1f.protocol.PaymentAddressMessage\"\x00\x12P\n\x15GetNewShieldedAddress\x12\x16.protocol.EmptyMessage\x1a\x1d.protocol.ShieldedAddressInfo\"\x00\x12:\n\x06GetRcm\x12\x16.protocol.EmptyMessage\x1a\x16.protocol.BytesMessage\"\x00\x12q\n CreateShieldedContractParameters\x12(.protocol.PrivateShieldedTRC20Parameters\x1a!.protocol.ShieldedTRC20Parameters\"\x00\x12\x85\x01\n*CreateShieldedContractParametersWithoutAsk\x12\x32.protocol.PrivateShieldedTRC20ParametersWithoutAsk\x1a!.protocol.ShieldedTRC20Parameters\"\x00\x12u\n\'GetTriggerInputForShieldedTRC20Contract\x12\x30.protocol.ShieldedTRC20TriggerContractParameters\x1a\x16.protocol.BytesMessage\"\x00\x32\xf4\x02\n\x0eWalletSolidity\x12\x42\n\nGetAccount\x12\x1f.protocol.AccountAddressMessage\x1a\x11.protocol.Account\"\x00\x12\x42\n\x0cGetNowBlock2\x12\x16.protocol.EmptyMessage\x1a\x18.protocol.BlockExtention\"\x00\x12O\n\x14GetBlockByLatestNum2\x12\x17.protocol.NumberMessage\x1a\x1c.protocol.BlockListExtention\"\x00\x12\x45\n\x12GetTransactionById\x12\x16.protocol.BytesMessage\x1a\x15.protocol.Transaction\"\x00\x12\x42\n\rGetRewardInfo\x12\x16.protocol.BytesMessage\x1a\x17.protocol.NumberMessage\"\x00\x62\x06proto3')



_EMPTYMESSAGE = DESCRIPTOR.message_types_by_name['EmptyMessage']
_NUMBERMESSAGE = DESCRIPTOR.message_types_by_name['NumberMessage']
_BYTESMESSAGE = DESCRIPTOR.message_types_by_name['BytesMessage']
_BLOCKLIMIT = DESCRIPTOR.message_types_by_name['BlockLimit']
_ACCOUNTADDRESSMESSAGE = DESCRIPTOR.message_types_by_name['AccountAddressMessage']
_ACCOUNTIDMESSAGE = DESCRIPTOR.message_types_by_name['AccountIdMessage']
_PAGINATEDMESSAGE = DESCRIPTOR.message_types_by_name['PaginatedMessage']
_EASYTRANSFERMESSAGE = DESCRIPTOR.message_types_by_name['EasyTransferMessage']
_EASYTRANSFERASSETMESSAGE = DESCRIPTOR.message_types_by_name['EasyTransferAssetMessage']
_EASYTRANSFERBYPRIVATEMESSAGE = DESCRIPTOR.message_types_by_name['EasyTransferByPrivateMessage']
_EASYTRANSFERASSETBYPRIVATEMESSAGE = DESCRIPTOR.message_types_by_name['EasyTransferAssetByPrivateMessage']
_VIEWINGKEYMESSAGE = DESCRIPTOR.message_types_by_name['ViewingKeyMessage']
_SHIELDEDADDRESSINFO = DESCRIPTOR.message_types_by_name['ShieldedAddressInfo']
_PAYMENTADDRESSMESSAGE = DESCRIPTOR.message_types_by_name['PaymentAddressMessage']
_DIVERSIFIERMESSAGE = DESCRIPTOR.message_types_by_name['DiversifierMessage']
_EXPANDEDSPENDINGKEYMESSAGE = DESCRIPTOR.message_types_by_name['ExpandedSpendingKeyMessage']
_INCOMINGVIEWINGKEYMESSAGE = DESCRIPTOR.message_types_by_name['IncomingViewingKeyMessage']
_INCOMINGVIEWINGKEYDIVERSIFIERMESSAGE = DESCRIPTOR.message_types_by_name['IncomingViewingKeyDiversifierMessage']
_RECEIVENOTE = DESCRIPTOR.message_types_by_name['ReceiveNote']
_SPENDNOTETRC20 = DESCRIPTOR.message_types_by_name['SpendNoteTRC20']
_PRIVATESHIELDEDTRC20PARAMETERS = DESCRIPTOR.message_types_by_name['PrivateShieldedTRC20Parameters']
_PRIVATESHIELDEDTRC20PARAMETERSWITHOUTASK = DESCRIPTOR.message_types_by_name['PrivateShieldedTRC20ParametersWithoutAsk']
_SPENDDESCRIPTION = DESCRIPTOR.message_types_by_name['SpendDescription']
_RECEIVEDESCRIPTION = DESCRIPTOR.message_types_by_name['ReceiveDescription']
_SHIELDEDTRC20PARAMETERS = DESCRIPTOR.message_types_by_name['ShieldedTRC20Parameters']
_IVKDECRYPTTRC20PARAMETERS = DESCRIPTOR.message_types_by_name['IvkDecryptTRC20Parameters']
_OVKDECRYPTTRC20PARAMETERS = DESCRIPTOR.message_types_by_name['OvkDecryptTRC20Parameters']
_NFTRC20PARAMETERS = DESCRIPTOR.message_types_by_name['NfTRC20Parameters']
_SHIELDEDTRC20TRIGGERCONTRACTPARAMETERS = DESCRIPTOR.message_types_by_name['ShieldedTRC20TriggerContractParameters']
EmptyMessage = _reflection.GeneratedProtocolMessageType('EmptyMessage', (_message.Message,), {
  'DESCRIPTOR' : _EMPTYMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.EmptyMessage)
  })
_sym_db.RegisterMessage(EmptyMessage)

NumberMessage = _reflection.GeneratedProtocolMessageType('NumberMessage', (_message.Message,), {
  'DESCRIPTOR' : _NUMBERMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.NumberMessage)
  })
_sym_db.RegisterMessage(NumberMessage)

BytesMessage = _reflection.GeneratedProtocolMessageType('BytesMessage', (_message.Message,), {
  'DESCRIPTOR' : _BYTESMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.BytesMessage)
  })
_sym_db.RegisterMessage(BytesMessage)

BlockLimit = _reflection.GeneratedProtocolMessageType('BlockLimit', (_message.Message,), {
  'DESCRIPTOR' : _BLOCKLIMIT,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.BlockLimit)
  })
_sym_db.RegisterMessage(BlockLimit)

AccountAddressMessage = _reflection.GeneratedProtocolMessageType('AccountAddressMessage', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTADDRESSMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.AccountAddressMessage)
  })
_sym_db.RegisterMessage(AccountAddressMessage)

AccountIdMessage = _reflection.GeneratedProtocolMessageType('AccountIdMessage', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTIDMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.AccountIdMessage)
  })
_sym_db.RegisterMessage(AccountIdMessage)

PaginatedMessage = _reflection.GeneratedProtocolMessageType('PaginatedMessage', (_message.Message,), {
  'DESCRIPTOR' : _PAGINATEDMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.PaginatedMessage)
  })
_sym_db.RegisterMessage(PaginatedMessage)

EasyTransferMessage = _reflection.GeneratedProtocolMessageType('EasyTransferMessage', (_message.Message,), {
  'DESCRIPTOR' : _EASYTRANSFERMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.EasyTransferMessage)
  })
_sym_db.RegisterMessage(EasyTransferMessage)

EasyTransferAssetMessage = _reflection.GeneratedProtocolMessageType('EasyTransferAssetMessage', (_message.Message,), {
  'DESCRIPTOR' : _EASYTRANSFERASSETMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.EasyTransferAssetMessage)
  })
_sym_db.RegisterMessage(EasyTransferAssetMessage)

EasyTransferByPrivateMessage = _reflection.GeneratedProtocolMessageType('EasyTransferByPrivateMessage', (_message.Message,), {
  'DESCRIPTOR' : _EASYTRANSFERBYPRIVATEMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.EasyTransferByPrivateMessage)
  })
_sym_db.RegisterMessage(EasyTransferByPrivateMessage)

EasyTransferAssetByPrivateMessage = _reflection.GeneratedProtocolMessageType('EasyTransferAssetByPrivateMessage', (_message.Message,), {
  'DESCRIPTOR' : _EASYTRANSFERASSETBYPRIVATEMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.EasyTransferAssetByPrivateMessage)
  })
_sym_db.RegisterMessage(EasyTransferAssetByPrivateMessage)

ViewingKeyMessage = _reflection.GeneratedProtocolMessageType('ViewingKeyMessage', (_message.Message,), {
  'DESCRIPTOR' : _VIEWINGKEYMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ViewingKeyMessage)
  })
_sym_db.RegisterMessage(ViewingKeyMessage)

ShieldedAddressInfo = _reflection.GeneratedProtocolMessageType('ShieldedAddressInfo', (_message.Message,), {
  'DESCRIPTOR' : _SHIELDEDADDRESSINFO,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ShieldedAddressInfo)
  })
_sym_db.RegisterMessage(ShieldedAddressInfo)

PaymentAddressMessage = _reflection.GeneratedProtocolMessageType('PaymentAddressMessage', (_message.Message,), {
  'DESCRIPTOR' : _PAYMENTADDRESSMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.PaymentAddressMessage)
  })
_sym_db.RegisterMessage(PaymentAddressMessage)

DiversifierMessage = _reflection.GeneratedProtocolMessageType('DiversifierMessage', (_message.Message,), {
  'DESCRIPTOR' : _DIVERSIFIERMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.DiversifierMessage)
  })
_sym_db.RegisterMessage(DiversifierMessage)

ExpandedSpendingKeyMessage = _reflection.GeneratedProtocolMessageType('ExpandedSpendingKeyMessage', (_message.Message,), {
  'DESCRIPTOR' : _EXPANDEDSPENDINGKEYMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ExpandedSpendingKeyMessage)
  })
_sym_db.RegisterMessage(ExpandedSpendingKeyMessage)

IncomingViewingKeyMessage = _reflection.GeneratedProtocolMessageType('IncomingViewingKeyMessage', (_message.Message,), {
  'DESCRIPTOR' : _INCOMINGVIEWINGKEYMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.IncomingViewingKeyMessage)
  })
_sym_db.RegisterMessage(IncomingViewingKeyMessage)

IncomingViewingKeyDiversifierMessage = _reflection.GeneratedProtocolMessageType('IncomingViewingKeyDiversifierMessage', (_message.Message,), {
  'DESCRIPTOR' : _INCOMINGVIEWINGKEYDIVERSIFIERMESSAGE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.IncomingViewingKeyDiversifierMessage)
  })
_sym_db.RegisterMessage(IncomingViewingKeyDiversifierMessage)

ReceiveNote = _reflection.GeneratedProtocolMessageType('ReceiveNote', (_message.Message,), {
  'DESCRIPTOR' : _RECEIVENOTE,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ReceiveNote)
  })
_sym_db.RegisterMessage(ReceiveNote)

SpendNoteTRC20 = _reflection.GeneratedProtocolMessageType('SpendNoteTRC20', (_message.Message,), {
  'DESCRIPTOR' : _SPENDNOTETRC20,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.SpendNoteTRC20)
  })
_sym_db.RegisterMessage(SpendNoteTRC20)

PrivateShieldedTRC20Parameters = _reflection.GeneratedProtocolMessageType('PrivateShieldedTRC20Parameters', (_message.Message,), {
  'DESCRIPTOR' : _PRIVATESHIELDEDTRC20PARAMETERS,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.PrivateShieldedTRC20Parameters)
  })
_sym_db.RegisterMessage(PrivateShieldedTRC20Parameters)

PrivateShieldedTRC20ParametersWithoutAsk = _reflection.GeneratedProtocolMessageType('PrivateShieldedTRC20ParametersWithoutAsk', (_message.Message,), {
  'DESCRIPTOR' : _PRIVATESHIELDEDTRC20PARAMETERSWITHOUTASK,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.PrivateShieldedTRC20ParametersWithoutAsk)
  })
_sym_db.RegisterMessage(PrivateShieldedTRC20ParametersWithoutAsk)

SpendDescription = _reflection.GeneratedProtocolMessageType('SpendDescription', (_message.Message,), {
  'DESCRIPTOR' : _SPENDDESCRIPTION,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.SpendDescription)
  })
_sym_db.RegisterMessage(SpendDescription)

ReceiveDescription = _reflection.GeneratedProtocolMessageType('ReceiveDescription', (_message.Message,), {
  'DESCRIPTOR' : _RECEIVEDESCRIPTION,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ReceiveDescription)
  })
_sym_db.RegisterMessage(ReceiveDescription)

ShieldedTRC20Parameters = _reflection.GeneratedProtocolMessageType('ShieldedTRC20Parameters', (_message.Message,), {
  'DESCRIPTOR' : _SHIELDEDTRC20PARAMETERS,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ShieldedTRC20Parameters)
  })
_sym_db.RegisterMessage(ShieldedTRC20Parameters)

IvkDecryptTRC20Parameters = _reflection.GeneratedProtocolMessageType('IvkDecryptTRC20Parameters', (_message.Message,), {
  'DESCRIPTOR' : _IVKDECRYPTTRC20PARAMETERS,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.IvkDecryptTRC20Parameters)
  })
_sym_db.RegisterMessage(IvkDecryptTRC20Parameters)

OvkDecryptTRC20Parameters = _reflection.GeneratedProtocolMessageType('OvkDecryptTRC20Parameters', (_message.Message,), {
  'DESCRIPTOR' : _OVKDECRYPTTRC20PARAMETERS,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.OvkDecryptTRC20Parameters)
  })
_sym_db.RegisterMessage(OvkDecryptTRC20Parameters)

NfTRC20Parameters = _reflection.GeneratedProtocolMessageType('NfTRC20Parameters', (_message.Message,), {
  'DESCRIPTOR' : _NFTRC20PARAMETERS,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.NfTRC20Parameters)
  })
_sym_db.RegisterMessage(NfTRC20Parameters)

ShieldedTRC20TriggerContractParameters = _reflection.GeneratedProtocolMessageType('ShieldedTRC20TriggerContractParameters', (_message.Message,), {
  'DESCRIPTOR' : _SHIELDEDTRC20TRIGGERCONTRACTPARAMETERS,
  '__module__' : 'graphsenselib.ingest.tron.grpc.api.tron_api_pb2'
  # @@protoc_insertion_point(class_scope:protocol.ShieldedTRC20TriggerContractParameters)
  })
_sym_db.RegisterMessage(ShieldedTRC20TriggerContractParameters)

_WALLET = DESCRIPTOR.services_by_name['Wallet']
_WALLETSOLIDITY = DESCRIPTOR.services_by_name['WalletSolidity']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EMPTYMESSAGE._serialized_start=266
  _EMPTYMESSAGE._serialized_end=280
  _NUMBERMESSAGE._serialized_start=282
  _NUMBERMESSAGE._serialized_end=310
  _BYTESMESSAGE._serialized_start=312
  _BYTESMESSAGE._serialized_end=341
  _BLOCKLIMIT._serialized_start=343
  _BLOCKLIMIT._serialized_end=389
  _ACCOUNTADDRESSMESSAGE._serialized_start=391
  _ACCOUNTADDRESSMESSAGE._serialized_end=431
  _ACCOUNTIDMESSAGE._serialized_start=433
  _ACCOUNTIDMESSAGE._serialized_end=463
  _PAGINATEDMESSAGE._serialized_start=465
  _PAGINATEDMESSAGE._serialized_end=514
  _EASYTRANSFERMESSAGE._serialized_start=516
  _EASYTRANSFERMESSAGE._serialized_end=592
  _EASYTRANSFERASSETMESSAGE._serialized_start=594
  _EASYTRANSFERASSETMESSAGE._serialized_end=692
  _EASYTRANSFERBYPRIVATEMESSAGE._serialized_start=694
  _EASYTRANSFERBYPRIVATEMESSAGE._serialized_end=779
  _EASYTRANSFERASSETBYPRIVATEMESSAGE._serialized_start=781
  _EASYTRANSFERASSETBYPRIVATEMESSAGE._serialized_end=888
  _VIEWINGKEYMESSAGE._serialized_start=890
  _VIEWINGKEYMESSAGE._serialized_end=933
  _SHIELDEDADDRESSINFO._serialized_start=936
  _SHIELDEDADDRESSINFO._serialized_end=1094
  _PAYMENTADDRESSMESSAGE._serialized_start=1096
  _PAYMENTADDRESSMESSAGE._serialized_end=1198
  _DIVERSIFIERMESSAGE._serialized_start=1200
  _DIVERSIFIERMESSAGE._serialized_end=1231
  _EXPANDEDSPENDINGKEYMESSAGE._serialized_start=1233
  _EXPANDEDSPENDINGKEYMESSAGE._serialized_end=1300
  _INCOMINGVIEWINGKEYMESSAGE._serialized_start=1302
  _INCOMINGVIEWINGKEYMESSAGE._serialized_end=1342
  _INCOMINGVIEWINGKEYDIVERSIFIERMESSAGE._serialized_start=1345
  _INCOMINGVIEWINGKEYDIVERSIFIERMESSAGE._serialized_end=1474
  _RECEIVENOTE._serialized_start=1476
  _RECEIVENOTE._serialized_end=1519
  _SPENDNOTETRC20._serialized_start=1521
  _SPENDNOTETRC20._serialized_end=1623
  _PRIVATESHIELDEDTRC20PARAMETERS._serialized_start=1626
  _PRIVATESHIELDEDTRC20PARAMETERS._serialized_end=1911
  _PRIVATESHIELDEDTRC20PARAMETERSWITHOUTASK._serialized_start=1914
  _PRIVATESHIELDEDTRC20PARAMETERSWITHOUTASK._serialized_end=2208
  _SPENDDESCRIPTION._serialized_start=2211
  _SPENDDESCRIPTION._serialized_end=2354
  _RECEIVEDESCRIPTION._serialized_start=2357
  _RECEIVEDESCRIPTION._serialized_end=2488
  _SHIELDEDTRC20PARAMETERS._serialized_start=2491
  _SHIELDEDTRC20PARAMETERS._serialized_end=2735
  _IVKDECRYPTTRC20PARAMETERS._serialized_start=2738
  _IVKDECRYPTTRC20PARAMETERS._serialized_end=2911
  _OVKDECRYPTTRC20PARAMETERS._serialized_start=2914
  _OVKDECRYPTTRC20PARAMETERS._serialized_end=3063
  _NFTRC20PARAMETERS._serialized_start=3066
  _NFTRC20PARAMETERS._serialized_end=3198
  _SHIELDEDTRC20TRIGGERCONTRACTPARAMETERS._serialized_start=3201
  _SHIELDEDTRC20TRIGGERCONTRACTPARAMETERS._serialized_end=3418
  _WALLET._serialized_start=3421
  _WALLET._serialized_end=13200
  _WALLETSOLIDITY._serialized_start=13203
  _WALLETSOLIDITY._serialized_end=13575
# @@protoc_insertion_point(module_scope)
